"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.precompile0b = void 0;
const util_1 = require("@ethereumjs/util");
const evm_js_1 = require("../evm.js");
const exceptions_js_1 = require("../exceptions.js");
const address = util_1.Address.fromString('0x000000000000000000000000000000000000000b');
async function precompile0b(opts) {
    const data = opts.data;
    const gasUsed = opts.common.param('gasPrices', 'beaconrootCost');
    if (opts._debug !== undefined) {
        opts._debug(`Run BEACONROOT (0x0B) precompile data=${(0, util_1.short)(opts.data)} length=${opts.data.length} gasLimit=${opts.gasLimit} gasUsed=${gasUsed}`);
    }
    if (opts.gasLimit < gasUsed) {
        if (opts._debug !== undefined) {
            opts._debug(`BEACONROOT (0x0B) failed: OOG`);
        }
        return (0, evm_js_1.OOGResult)(opts.gasLimit);
    }
    if (data.length < 32) {
        return {
            returnValue: new Uint8Array(0),
            executionGasUsed: gasUsed,
            exceptionError: new exceptions_js_1.EvmError(exceptions_js_1.ERROR.INVALID_INPUT_LENGTH),
        };
    }
    const timestampInput = (0, util_1.bytesToBigInt)(data.slice(0, 32));
    const historicalRootsLength = BigInt(opts.common.param('vm', 'historicalRootsLength'));
    const timestampIndex = timestampInput % historicalRootsLength;
    const recordedTimestamp = await opts._EVM.stateManager.getContractStorage(address, (0, util_1.setLengthLeft)((0, util_1.bigIntToBytes)(timestampIndex), 32));
    if ((0, util_1.bytesToBigInt)(recordedTimestamp) !== timestampInput) {
        return {
            executionGasUsed: gasUsed,
            returnValue: (0, util_1.zeros)(32),
        };
    }
    const timestampExtended = timestampIndex + historicalRootsLength;
    const returnData = (0, util_1.setLengthLeft)(await opts._EVM.stateManager.getContractStorage(address, (0, util_1.setLengthLeft)((0, util_1.bigIntToBytes)(timestampExtended), 32)), 32);
    if (opts._debug !== undefined) {
        opts._debug(`BEACONROOT (0x0B) return data=${(0, util_1.short)(returnData)}`);
    }
    return {
        executionGasUsed: gasUsed,
        returnValue: returnData,
    };
}
exports.precompile0b = precompile0b;
//# sourceMappingURL=0b-beaconroot.js.map